clc
warning off
%% Parameters
magnitude = 450;
random_def = 0;
comp = 1;
%% Load the aligned models
if ~exist('alignedModels_lin','var')
    load('data/alignedModels_bh_1779_lin.mat')
end
%% Build the average model
m = mean(alignedModels_lin,2);
% avgModel = reshape(m,3,20112/3)';
load('data/avgModel_bh_1779_NE.mat')
plot_mesh(avgModel,compute_delaunay(avgModel));
pause
close

%% Center the data
alignedModels_centerd = alignedModels_lin - repmat(m,1,1779);

%% Apply DL
if ~exist('components','var')
    %% General
    paramDL.verbose = false;
    %% Iterations
    paramDL.numThreads=8; % number of threads
    paramDL.iter=-5;  % until convergence
    %% Learning mode
    paramDL.mode=2; % elastic net
    %% Dictionary (components)
    paramDL.modeD = 0;
    paramDL.K = 300;        % Number of dictionary atoms
    paramDL.lambda = 0.1;    % Regularization l1
    paramDL.lambda2 = 100;    % Regularization l2
    disp('Computing Dictionary')
    [weights, components] = DL_3dmm(alignedModels_centerd,paramDL);
end
%% Perform Random Deformations
if random_def
    alpha = rand(size(components,2),1);
    alpha = rescale(alpha, -magnitude, magnitude);
else
    alpha = zeros(size(components,2),1);
    alpha(comp) = magnitude;
end
%% sum_1:n(alpha_i*w_i): alpha is column vector
dim = size(components,1)/3;
alpha_full = repmat(alpha',size(components,1),1);
tmp_eigen = alpha_full.*components;
sumVec = sum(tmp_eigen,2);
sumMat = reshape(sumVec',3,dim);

%%  S = m + sum_1:n(alpha_i*w_i)
defShape = avgModel + sumMat';

subplot(1,3,1)
plot_mesh(avgModel,compute_delaunay(avgModel));
title('Average Model')
subplot(1,3,2)
plot_mesh(defShape,compute_delaunay(defShape));
title('Deformed Model')
subplot(1,3,3)
show3DFaceHeatMap(avgModel,defShape)
title('Heatmap')
pause
close
